;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================

ORG 5014

@Font=1E00
@DFile=400C
@HRDFile=8100			;Hires DFile
@WorkArea=8000			;Storage AND scratch area

;==========================
; Variables
;==========================
@X1Y1				;PLOT coordinates
	00 00
@Var				;Variable
	00
;==========================
; JumpTable
;==========================
	JR WRX
	JP LoRes
	JP CLR
	JP Invert
	JP Copy
	JP Plot
	JP XPlot
	JP Point
	JP SaveLine
	JP LoadLine
	JP VFlip
	JP HFlip
;================================
;Wilf Rigter's Video Driver
;================================
@WRX
	LD IX,HR		;Starts the hires mode when JP (IX) is made
	RET
;================================
@HLine
	LD R,A 			;Load LSB to R register which is RFSH address LSB
	00 00 00 00		;32 NOPs = 32x8 bits = 256 pixels each line
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	00 00 00 00
	RET NZ 			;Always returns because Z flag=0
;================================
@HR
	LD B,06			;Delay sets the left edge of the screen  
	DJNZ -2			;Delay loop
	00 00 00 00 00		;Fine tuning
	DEC B			;Make Z flag=0
	LD HL,HRDFile 		;HL is the HDFile address pointer for each line
				;HL should be divisible by 32
	LD DE,0020 		;32 bytes offset is added to HL for next line
	LD B,C0	 		;192 lines per hires screen
@HRLoop1
	LD A,H 			;Get HDFile address MSB from HL
	LD I,A 			;Load MSB into I register which is RFSH address MSB
	LD A,L 			;Get HDFile address LSB from HL
	CALL D03F 		;CALL HLine+8000
	ADD HL,DE 		;Add 32 to HL to point to next line
	DEC B 			;Decrement line counter
	JP NZ HRLoop1 		;Test for last line
	CALL 0292		;Display-3, Return to application program
	CALL 0220		;PUSH register then do VSYNC and get KEYBD
	CALL 0F46		;Check break (space) key
	LD A,1E			;The I register is restored with the MSB address of
	LD I,A 			;of the ROM pattern table in case of BREAK key down
	JR NC +4		;IF break then restore Sinclair video
	LD IX,HR 		;Reload HR vector if no BREAK
	JP 02A4			;Return to BASIC and other application programs
;================================
;Restore Sinclair display driver
;================================
@LoRes
	HALT 			;Sync the display
	LD A,(4034) 		;Get Frames
	LD C,A 
	LD A,(4034) 		;Get Frames
	CP C 
	JR Z -6
	LD A,1E 		;Restore Sinclair video
	LD I,A 
	LD IX,0281 		;Set IX vector to Sinclair display
	LD BC,0000 		;Return 0 to BASIC
	RET
;================================
;HiRes CLS
;================================
@CLR
	LD HL,HRDFile
	LD (HL),00 
	LD D,H 
	LD E,L 
	INC DE 
	LD BC,17FF 
	LDIR 
	RET
;================================
;Invert
;================================
@Invert
	LD HL,HLine+2 
	LD A,(HL) 
	ADD A,80 
	LD (HL),A 
	LD D,H 
	LD E,L 
	INC DE 
	LD BC,001F 
	LDIR 
	RET
;================================
; Test XY for bounds
; On exit...
;  B=Ycoord
;  C=Xcoord
;================================
@TestXY
	LD BC,(X1Y1)		;
	LD A,C			;C=Xcoord
	SUB 80			;Xcoord-128
	JR NC ErrorH		;Xcoord>128, Error H report
	LD A,B			;B=Ycoord
	SUB 60			;Ycoord-96
	RET C
@ErrorV
	CALL LoRes
	RST 08			;ERROR-1
	1E			;Error Code'V', Vertical value out of range
@ErrorH
	CALL LoRes
	RST 08			;ERROR-1
	10			;Error Code'H', Horizontal value out of range
;================================
; PLOT/UNPLOT routines
;================================
@Plot
	LD HL,SetBit		;Set bits table
	JR +3
@XPlot
	LD HL,ResBit		;Reset bits table
	LD (SetOP+1),HL
@PlotPx
	CALL TestXY		;Test coords
	CALL GetAddr		;HL=HRDFile addr, BC=Bit to set or reset
	EX DE,HL		;DE=HRDFile addr
@SetOP
	LD HL,0000		;Plot/Unplot opcodes table
	ADD HL,BC		;Point to opcode in table
	LD A,(HL)		;Get opcode
	LD (SetRes+1),A		;Set bit X instruction
	DEC HL			;Decrease table pointer
	LD A,(HL)		;Get opcode
	LD (SetRes+3),A		;Set bit X+1 instruction
	EX DE,HL		;HL=HRDFile addr
@SetRes
	SET 0,(HL)		;Set/Res bit X in (HL)
	SET 1,(HL)		;Set/Res bit X+1 in (HL)
	LD A,(HL)		;Get (HL)
;	AND A			;Clear CY
	LD BC,0020		;32 bytes/row
	SBC HL,BC		;HL=Top addr
	LD (HL),A		;Set top addr
	LD C,B			;Return 0 to BASIC
	RET
@SetBit
	C6
	CE
	D6
	DE
	E6
	EE
	F6
	FE
@ResBit
	86
	8E
	96
	9E
	A6
	AE
	B6
	BE
;================================
; Locate HRDFile address
; On entry...
;  B=Ycoord
;  C=Xcoord
; On exit...
;  HL=HRDFile addr
;  BC=Bit to set or reset in
;     HL, 0 to 7
;================================
@GetAddr
	LD A,C0 		;192 Rows
	RL B			;Double the Ycoord
	SUB B 			;Subtract Ycoord
	LD B,A 			;B=A=192-Ycoord
	RL C			;Double the Xcoord
	LD A,C			;A=C=Xcoord
	SRL A 			;Convert Xcoord to byte coord
	SRL A 			;A=A/4
	SRL A 			;A=A/8
	LD D,C 			;D=Xcoord
	LD C,A 			;C=Byte coord
	PUSH DE 		;PUSH Xcoord
@RowsX32
	LD H,00 		
	LD D,H 			
	LD L,B 			;L=Row #
	LD B,05 		;Multiply HL*32
@HLx32
	SLA L 			;L=L*2
	RL H 			;Move CY to H
	DJNZ HLx32
	ADD HL,BC 		;ADD X byte coord
	LD BC,HRDFile
	ADD HL,BC 		;HL=Addr of X,Y coordinates 
	POP AF 			;POP X pixel coord
	CPL 			;Byte mask
	AND 07 			;
	LD B,00
        LD C,A			;BC=Bit in byte to Plot/UnPlot
	RET
;================================
; Point routine
;
; USAGE:
;
;  PRINT USR POINT;X,Y
;
;   X=Xcoord
;   Y=Ycoord
;
;   Point value returned to BASIC
;================================
@Point
	CALL TestXY		;Test coords, B=Ycoord C=Xcoord
	LD A,5F
	SUB B			;A=95-Ycoord
	LD B,A 			;B=95-Ycoord
	SLA B			;Double Ycoord
	LD A,C 			;A=Xcoord
	SRL A 			;Convert Xcoord to Chr coord
	SRL A 
	LD D,C			;D=Xcoord
	SLA D			;Double Xcoord
	PUSH DE			;PUSH Xcoord
	LD C,A 			;C=Column offset
	LD H,00 		;Locate HRDFile address
	LD D,H 			;D=0
	LD L,B 			;Ycoord
	LD B,05 
	SLA L 
	RL H 
	DJNZ -6
	ADD HL,BC 		;Add column offset
	LD BC,HRDFile		;BC=HRDFile
	ADD HL,BC 		;HL=HRDFile+Offset
	POP AF 			;POP Xcoord
	CPL 
	AND 07
	INC A
	LD B,A 			;B=Pixel position in byte
	LD A,(HL) 		;Get byte to Test
	RRCA 			;Move pixel to Test to CY
	DJNZ -3
	RL B			;Move CY B, B=0 or 1
	LD C,B			;Return point value to BASIC
	LD B,00
	RET
;==========================
;Copy DFile to HRDFile
;==========================
@Copy
	LD HL,HRDFile		;HL=HRDFile
	LD DE,(DFile)		;DE=DFile
	INC DE			;DE=DFile+1
	LD A,(Var)		;Get row counter
	LD B,A			;Set row counter
@TrLoop
	XOR A			;A=0
	LD (Tr01+2),A		;Initialize Graphics opcode
	LD A,(DE)		;Get chr. to transfer
	CP 76			;DFile EOL marker?
	JR Z Tr02		;Yes, start next row
	PUSH HL			;PUSH HRDFile addr
	PUSH DE			;PUSH DFile addr
	PUSH BC			;PUSH row counter
	LD DE,Font		;Font patterns
	CP 40			;Normal or Graphic character?
	LD C,A			;C=Chr code
	JR C +7			;Code < 64?
	RES 7,C			;Clear the graphic chr bit
	LD A,2F			;CPL opcode
	LD (Tr01+2),A		;Set Graphics opcode to CPL instruction
	LD B,00			;Pattern location=DE+8*Chr Code
	XOR A			;Clear CY 
	RL C			;2 x code
	RL B			;
	RL C			;4 x code
	RL B			; 
	RL C			;8 x code
	RL B 			;
	EX DE,HL		;HL=Pattern addr, DE=HRDFile addr
	ADD HL,BC		;Add offset to pattern table address
	EX DE,HL		;HL=HRDFile addr, DE=Pattern addr
	LD B,08			;8 Rows/Character
@Tr01
	LD A,(DE)		;A=Bit Pattern 
	PUSH BC			;PUSH counter
	00			;NOP or CPL instruction
	LD (HL),A		;Transfer pattern to HRDFile
	LD BC,0020		;32 Bytes/row
	ADD HL,BC 		;Addr of next row
	POP BC 			;POP counter
	INC DE			;Next Pattern addr 
	DJNZ Tr01		;Transfer 8 bytes/character
	POP BC			;POP rows done counter
	POP DE 			;POP DFile addr
	POP HL 			;POP HRDFile addr
	INC DE 			;Next DFile addr
	INC HL 			;Next HRDFile addr
	JR TrLoop		;Do next chr
@Tr02
	INC DE 			;Skip over EOL marker in DFile
	PUSH BC			;PUSH counter
	LD BC,00E0		;BC=Bytes/HRDFile row, 224
	ADD HL,BC		;HL=Starting address of next HRDFile line
	POP BC			;POP counter
	DJNZ TrLoop		;If not 0 then start next row
	LD C,B			;Return 0 to BASIC
	RET
;================================
; Save line
;================================
@SaveLine
	LD HL,HRDFile		;Set source
	LD DE,WorkArea		;Set destination
	LD BC,0100		;Byte count
	LDIR			;Copy bytes
	RET
;================================
; Load line
;================================
@LoadLine
	LD HL,WorkArea		;Set source
	LD DE,HRDFile		;Set destination
	JR SaveLine+6
;================================
; WRX Vertical flip
;================================
@VFlip
	LD HL,HRDFile		;Start of HRDFile
	LD (WorkArea),HL		;Save top row addr
	LD BC,17E0		;Offset to bottom row
	ADD HL,BC		;HL=Address of bottom row.
	LD B,60			;Rows to process counter
	LD A,20			;Constant
@VFLoop
	PUSH BC			;PUSH counter
	LD (WorkArea+2),HL	;Save bottom row addr
	LD HL,(WorkArea)	;Transfer top row data to storage area
	LD DE,WorkArea+4
	LD BC,0020
	LDIR
	LD HL,(WorkArea)	;Transfer bottom row data to top row address
	EX DE,HL
	LD HL,(WorkArea+2)
	LD C,A
	LDIR
	LD (WorkArea),DE	;Save top row address
	LD HL,WorkArea+4	;Transfer top row data to bottom row address
	LD DE,(WorkArea+2)
	LD C,A
	LDIR
	LD C,A			;Calculate address of next bottom row
	LD HL,(WorkArea+2)
	AND A			;Clear CY
	SBC HL,BC
	POP BC			;POP row counter
	DJNZ VFLoop		;Repeat if NZ
	LD C,B			;Return 0 to BASIC
	RET
;================================
; WRX Horizontal flip
;================================
@HFlip
	LD HL,HRDFile		;Left column
	LD DE,811F 		;Right column
	LD B,C0			;Row counter 
@FlipOuter
	PUSH BC			;PUSH row counter
	LD B,10			;Column counter
@FlipInner
	PUSH BC			;PUSH column counter
	LD A,(HL) 		;Get left column value
	LD BC,0008
@FlipLeft
	RLA			;Flip left value bit pattern
	RR B
	DEC C
	JR NZ FlipLeft
	LD A,(DE)		;Get right column byte
	EX DE,HL		;HL=right, DE=left
	LD (HL),B 		;Copy flipped left value right
	LD BC,0008
@FlipRite
	RLA			;Flip right value bit pattern
	RR B
	DEC C
	JR NZ FlipRite
	EX DE,HL		;HL=left, DE=right
	LD (HL),B 		;Copy right flipped value left
	INC HL 			;Left+1
	DEC DE 			;Right-1
	POP BC			;POP column counter
	DJNZ FlipInner		;Repeat x 16 columns
	LD C,10 		;Start next row
	ADD HL,BC		;Next left row
	EX DE,HL		;DE=left
	LD H,D
	LD L,E
	LD C,1F
	ADD HL,BC		;Next right row
	EX DE,HL		;HL=left, DE=right
	POP BC			;POP row counter
	DJNZ FlipOuter		;Repeat x 192 rows
	LD C,B			;Return 0 to BASIC
	RET
